@@ -36,7 +36,7 @@ def getopenfiles(out):
 def check_open_files(config):
     out2 = py.process.cmdexec("lsof -p %d" % pid)
     lines2 = getopenfiles(out2)
-    if len(lines2) > config._numfiles + 1:
+    if len(lines2) > config._numfiles + 3:
         error = []
         error.append("***** %s FD leackage detected" %
     (len(lines2)-config._numfiles))
@@ -46,6 +46,7 @@ def check_open_files(config):
         config._numfiles = len(lines2)
         raise AssertionError("\n".join(error))
 
+@pytest.mark.tryfirst # XXX rather do item.addfinalizer
 def pytest_runtest_setup(item):
     item._oldir = py.path.local()
 
@@ -63,9 +64,10 @@ def pytest_generate_tests(metafunc):
         for name, l in multi.kwargs.items():
             for val in l:
                 metafunc.addcall(funcargs={name: val})
-    elif 'anypython' in metafunc.funcargnames:
-        for name in ('python2.4', 'python2.5', 'python2.6',
-                     'python2.7', 'python3.1', 'pypy', 'jython'):
+    elif 'anypython' in metafunc.fixturenames:
+        for name in ('python2.5', 'python2.6',
+                     'python2.7', 'python3.2', "python3.3",
+                     'pypy', 'jython'):
             metafunc.addcall(id=name, param=name)
 
 # XXX copied from execnet's conftest.py - needs to be merged
@@ -75,6 +77,9 @@ winpymap = {
     'python2.5': r'C:\Python25\python.exe',
     'python2.4': r'C:\Python24\python.exe',
     'python3.1': r'C:\Python31\python.exe',
+    'python3.2': r'C:\Python32\python.exe',
+    'python3.3': r'C:\Python33\python.exe',
+    'python3.4': r'C:\Python34\python.exe',
 }
 
 def getexecutable(name, cache={}):
