@@ -3,9 +3,20 @@ import sys
 import py, pytest
 import _pytest.assertion as plugin
 from _pytest.assertion import reinterpret, util
-
 needsnewassert = pytest.mark.skipif("sys.version_info < (2,6)")
 
+
+@pytest.fixture
+def mock_config():
+    class Config(object):
+        verbose = False
+        def getoption(self, name):
+            if name == 'verbose':
+                return self.verbose
+            raise KeyError('Not mocked out: %s' % name)
+    return Config()
+
+
 def interpret(expr):
     return reinterpret.reinterpret(expr, py.code.Frame(sys._getframe(1)))
 
@@ -32,8 +43,11 @@ class TestBinReprIntegration:
             "*test_check*PASS*",
         ])
 
-def callequal(left, right):
-    return plugin.pytest_assertrepr_compare('==', left, right)
+def callequal(left, right, verbose=False):
+    config = mock_config()
+    config.verbose = verbose
+    return plugin.pytest_assertrepr_compare(config, '==', left, right)
+
 
 class TestAssert_reprcompare:
     def test_different_types(self):
@@ -48,6 +62,17 @@ class TestAssert_reprcompare:
         assert '- spam' in diff
         assert '+ eggs' in diff
 
+    def test_text_skipping(self):
+        lines = callequal('a'*50 + 'spam', 'a'*50 + 'eggs')
+        assert 'Skipping' in lines[1]
+        for line in lines:
+            assert 'a'*50 not in line
+
+    def test_text_skipping_verbose(self):
+        lines = callequal('a'*50 + 'spam', 'a'*50 + 'eggs', verbose=True)
+        assert '- ' + 'a'*50 + 'spam' in lines
+        assert '+ ' + 'a'*50 + 'eggs' in lines
+
     def test_multiline_text_diff(self):
         left = 'foo\nspam\nbar'
         right = 'foo\neggs\nbar'
@@ -69,10 +94,58 @@ class TestAssert_reprcompare:
         expl = callequal({'a': 0}, {'a': 1})
         assert len(expl) > 1
 
+    def test_dict_omitting(self):
+        lines = callequal({'a': 0, 'b': 1}, {'a': 1, 'b': 1})
+        assert lines[1].startswith('Omitting 1 identical item')
+        assert 'Common items' not in lines
+        for line in lines[1:]:
+            assert 'b' not in line
+
+    def test_dict_omitting_verbose(self):
+        lines = callequal({'a': 0, 'b': 1}, {'a': 1, 'b': 1}, verbose=True)
+        assert lines[1].startswith('Common items:')
+        assert 'Omitting' not in lines[1]
+        assert lines[2] == "{'b': 1}"
+
     def test_set(self):
         expl = callequal(set([0, 1]), set([0, 2]))
         assert len(expl) > 1
 
+    def test_frozenzet(self):
+        expl = callequal(frozenset([0, 1]), set([0, 2]))
+        assert len(expl) > 1
+
+    def test_Sequence(self):
+        col = py.builtin._tryimport(
+            "collections.abc",
+            "collections",
+            "sys")
+        if not hasattr(col, "MutableSequence"):
+            pytest.skip("cannot import MutableSequence")
+        MutableSequence = col.MutableSequence
+
+        class TestSequence(MutableSequence):  # works with a Sequence subclass
+            def __init__(self, iterable):
+                self.elements = list(iterable)
+
+            def __getitem__(self, item):
+                return self.elements[item]
+
+            def __len__(self):
+                return len(self.elements)
+
+            def __setitem__(self, item, value):
+                pass
+
+            def __delitem__(self, item):
+                pass
+
+            def insert(self, item, index):
+                pass
+
+        expl = callequal(TestSequence([0, 1]), list([0, 2]))
+        assert len(expl) > 1
+
     def test_list_tuples(self):
         expl = callequal([], [(1,2)])
         assert len(expl) > 1
@@ -103,6 +176,19 @@ class TestAssert_reprcompare:
         expl = ' '.join(callequal('foo', 'bar'))
         assert 'raised in repr()' not in expl
 
+def test_python25_compile_issue257(testdir):
+    testdir.makepyfile("""
+        def test_rewritten():
+            assert 1 == 2
+        # some comment
+    """)
+    result = testdir.runpytest()
+    assert result.ret == 1
+    result.stdout.fnmatch_lines("""
+            *E*assert 1 == 2*
+            *1 failed*
+    """)
+
 @needsnewassert
 def test_rewritten(testdir):
     testdir.makepyfile("""
@@ -111,8 +197,9 @@ def test_rewritten(testdir):
     """)
     assert testdir.runpytest().ret == 0
 
-def test_reprcompare_notin():
-    detail = plugin.pytest_assertrepr_compare('not in', 'foo', 'aaafoobbb')[1:]
+def test_reprcompare_notin(mock_config):
+    detail = plugin.pytest_assertrepr_compare(
+        mock_config, 'not in', 'foo', 'aaafoobbb')[1:]
     assert detail == ["'foo' is contained here:", '  aaafoobbb', '?    +++']
 
 @needsnewassert
@@ -164,7 +251,7 @@ def test_assert_compare_truncate_longmessage(testdir):
 
     result = testdir.runpytest()
     result.stdout.fnmatch_lines([
-        "*too verbose, truncated*",
+        "*truncated*use*-vv*",
     ])
 
 
@@ -275,3 +362,17 @@ def test_warn_missing(testdir):
     result.stderr.fnmatch_lines([
         "*WARNING*assert statements are not executed*",
     ])
+
+def test_recursion_source_decode(testdir):
+    testdir.makepyfile("""
+        def test_something():
+            pass
+    """)
+    testdir.makeini("""
+        [pytest]
+        python_files = *.py
+    """)
+    result = testdir.runpytest("--collect-only")
+    result.stdout.fnmatch_lines("""
+        <Module*>
+    """)
