@@ -1,3 +1,5 @@
+import os
+import stat
 import sys
 import zipfile
 import py
@@ -9,7 +11,7 @@ if sys.platform.startswith("java"):
     pytest.skip("assert rewrite does currently not work on jython")
 
 from _pytest.assertion import util
-from _pytest.assertion.rewrite import rewrite_asserts
+from _pytest.assertion.rewrite import rewrite_asserts, PYTEST_TAG
 
 
 def setup_module(mod):
@@ -106,7 +108,15 @@ class TestAssertionRewrite:
         assert getmsg(f) == "assert False"
         def f():
             assert a_global
-        assert getmsg(f, {"a_global" : False}) == "assert a_global"
+        assert getmsg(f, {"a_global" : False}) == "assert False"
+        def f():
+            assert sys == 42
+        assert getmsg(f, {"sys" : sys}) == "assert sys == 42"
+        def f():
+            assert cls == 42
+        class X(object):
+            pass
+        assert getmsg(f, {"cls" : X}) == "assert cls == 42"
 
     def test_assert_already_has_message(self):
         def f():
@@ -231,7 +241,7 @@ class TestAssertionRewrite:
     def test_attribute(self):
         class X(object):
             g = 3
-        ns = {"X" : X, "x" : X()}
+        ns = {"x" : X}
         def f():
             assert not x.g
         assert getmsg(f, ns) == """assert not 3
@@ -310,10 +320,22 @@ class TestRewriteOnImport:
     def test_pycache_is_a_file(self, testdir):
         testdir.tmpdir.join("__pycache__").write("Hello")
         testdir.makepyfile("""
-def test_rewritten():
-    assert "@py_builtins" in globals()""")
+            def test_rewritten():
+                assert "@py_builtins" in globals()""")
         assert testdir.runpytest().ret == 0
 
+    def test_pycache_is_readonly(self, testdir):
+        cache = testdir.tmpdir.mkdir("__pycache__")
+        old_mode = cache.stat().mode
+        cache.chmod(old_mode ^ stat.S_IWRITE)
+        testdir.makepyfile("""
+            def test_rewritten():
+                assert "@py_builtins" in globals()""")
+        try:
+            assert testdir.runpytest().ret == 0
+        finally:
+            cache.chmod(old_mode)
+
     def test_zipfile(self, testdir):
         z = testdir.tmpdir.join("myzip.zip")
         z_fn = str(z)
@@ -325,9 +347,9 @@ def test_rewritten():
             f.close()
         z.chmod(256)
         testdir.makepyfile("""
-import sys
-sys.path.append(%r)
-import test_gum.test_lizard""" % (z_fn,))
+            import sys
+            sys.path.append(%r)
+            import test_gum.test_lizard""" % (z_fn,))
         assert testdir.runpytest().ret == 0
 
     def test_readonly(self, testdir):
@@ -336,34 +358,44 @@ import test_gum.test_lizard""" % (z_fn,))
         py.builtin._totext("""
 def test_rewritten():
     assert "@py_builtins" in globals()
-""").encode("utf-8"), "wb")
+            """).encode("utf-8"), "wb")
+        old_mode = sub.stat().mode
         sub.chmod(320)
-        assert testdir.runpytest().ret == 0
+        try:
+            assert testdir.runpytest().ret == 0
+        finally:
+            sub.chmod(old_mode)
 
     def test_dont_write_bytecode(self, testdir, monkeypatch):
         testdir.makepyfile("""
-import os
-def test_no_bytecode():
-    assert "__pycache__" in __cached__
-    assert not os.path.exists(__cached__)
-    assert not os.path.exists(os.path.dirname(__cached__))""")
+            import os
+            def test_no_bytecode():
+                assert "__pycache__" in __cached__
+                assert not os.path.exists(__cached__)
+                assert not os.path.exists(os.path.dirname(__cached__))""")
         monkeypatch.setenv("PYTHONDONTWRITEBYTECODE", "1")
         assert testdir.runpytest().ret == 0
 
     @pytest.mark.skipif('"__pypy__" in sys.modules')
     def test_pyc_vs_pyo(self, testdir, monkeypatch):
         testdir.makepyfile("""
-import pytest
-def test_optimized():
-    "hello"
-    assert test_optimized.__doc__ is None""")
+            import pytest
+            def test_optimized():
+                "hello"
+                assert test_optimized.__doc__ is None"""
+        )
         p = py.path.local.make_numbered_dir(prefix="runpytest-", keep=None,
                                             rootdir=testdir.tmpdir)
         tmp = "--basetemp=%s" % p
         monkeypatch.setenv("PYTHONOPTIMIZE", "2")
+        monkeypatch.delenv("PYTHONDONTWRITEBYTECODE", raising=False)
         assert testdir.runpybin("py.test", tmp).ret == 0
+        tagged = "test_pyc_vs_pyo." + PYTEST_TAG
+        assert tagged + ".pyo" in os.listdir("__pycache__")
         monkeypatch.undo()
+        monkeypatch.delenv("PYTHONDONTWRITEBYTECODE", raising=False)
         assert testdir.runpybin("py.test", tmp).ret == 1
+        assert tagged + ".pyc" in os.listdir("__pycache__")
 
     def test_package(self, testdir):
         pkg = testdir.tmpdir.join("pkg")
@@ -379,3 +411,61 @@ def test_rewritten():
         b = content.encode("utf-8")
         testdir.tmpdir.join("test_newlines.py").write(b, "wb")
         assert testdir.runpytest().ret == 0
+
+
+class TestAssertionRewriteHookDetails(object):
+    def test_loader_is_package_false_for_module(self, testdir):
+        testdir.makepyfile(test_fun="""
+            def test_loader():
+                assert not __loader__.is_package(__name__)
+            """)
+        result = testdir.runpytest()
+        result.stdout.fnmatch_lines([
+            "* 1 passed*",
+        ])
+
+    def test_loader_is_package_true_for_package(self, testdir):
+        testdir.makepyfile(test_fun="""
+            def test_loader():
+                assert not __loader__.is_package(__name__)
+
+            def test_fun():
+                assert __loader__.is_package('fun')
+
+            def test_missing():
+                assert not __loader__.is_package('pytest_not_there')
+            """)
+        pkg = testdir.mkpydir('fun')
+        result = testdir.runpytest()
+        result.stdout.fnmatch_lines([
+            '* 3 passed*',
+        ])
+
+
+    @pytest.mark.skipif("sys.version_info[0] >= 3")
+    def test_assume_ascii(self, testdir):
+        content = "u'\xe2\x99\xa5'"
+        testdir.tmpdir.join("test_encoding.py").write(content, "wb")
+        res = testdir.runpytest()
+        assert res.ret != 0
+        assert "SyntaxError: Non-ASCII character" in res.stdout.str()
+
+
+    def test_write_pyc(self, testdir, tmpdir, monkeypatch):
+        from _pytest.assertion.rewrite import _write_pyc
+        from _pytest.assertion import AssertionState
+        try:
+            import __builtin__ as b
+        except ImportError:
+            import builtins as b
+        config = testdir.parseconfig([])
+        state = AssertionState(config, "rewrite")
+        source_path = tmpdir.ensure("source.py")
+        pycpath = tmpdir.join("pyc").strpath
+        assert _write_pyc(state, [1], source_path, pycpath)
+        def open(*args):
+            e = IOError()
+            e.errno = 10
+            raise e
+        monkeypatch.setattr(b, "open", open)
+        assert not _write_pyc(state, [1], source_path, pycpath)
