@@ -35,6 +35,33 @@ def test_setattr():
     monkeypatch.undo() # double-undo makes no modification
     assert A.x == 5
 
+class TestSetattrWithImportPath:
+    def test_string_expression(self, monkeypatch):
+        monkeypatch.setattr("os.path.abspath", lambda x: "hello2")
+        assert os.path.abspath("123") == "hello2"
+
+    def test_string_expression_class(self, monkeypatch):
+        monkeypatch.setattr("_pytest.config.Config", 42)
+        import _pytest
+        assert _pytest.config.Config == 42
+
+    def test_wrong_target(self, monkeypatch):
+        pytest.raises(TypeError, lambda: monkeypatch.setattr(None, None))
+
+    def test_unknown_import(self, monkeypatch):
+        pytest.raises(pytest.fail.Exception,
+                      lambda: monkeypatch.setattr("unkn123.classx", None))
+
+    def test_unknown_attr(self, monkeypatch):
+        pytest.raises(pytest.fail.Exception,
+                      lambda: monkeypatch.setattr("os.path.qweqwe", None))
+
+    def test_delattr(self, monkeypatch):
+        monkeypatch.delattr("os.path.abspath")
+        assert not hasattr(os.path, "abspath")
+        monkeypatch.undo()
+        assert os.path.abspath
+
 def test_delattr():
     class A:
         x = 1
@@ -193,3 +220,53 @@ def test_chdir_double_undo(mp, tmpdir):
     tmpdir.chdir()
     mp.undo()
     assert os.getcwd() == tmpdir.strpath
+
+def test_issue185_time_breaks(testdir):
+    testdir.makepyfile("""
+        import time
+        def test_m(monkeypatch):
+            def f():
+                raise Exception
+            monkeypatch.setattr(time, "time", f)
+    """)
+    result = testdir.runpytest()
+    result.stdout.fnmatch_lines("""
+        *1 passed*
+    """)
+
+
+
+class SampleNew(object):
+    @staticmethod
+    def hello():
+        return True
+
+
+class SampleNewInherit(SampleNew):
+    pass
+
+
+class SampleOld:
+    #oldstyle on python2
+    @staticmethod
+    def hello():
+        return True
+
+class SampleOldInherit(SampleOld):
+    pass
+
+
+@pytest.mark.parametrize('Sample', [
+    SampleNew, SampleNewInherit,
+    SampleOld, SampleOldInherit,
+], ids=['new', 'new-inherit', 'old', 'old-inherit'])
+def test_issue156_undo_staticmethod(Sample):
+    monkeypatch = MonkeyPatch()
+
+    monkeypatch.setattr(Sample, 'hello', None)
+    assert Sample.hello is None
+
+    monkeypatch.undo()
+    assert Sample.hello()
+
+
