@@ -35,6 +35,12 @@ def test_setup_func_with_setup_decorator():
     assert not l
 
 
+def test_setup_func_not_callable():
+    from _pytest.nose import call_optional
+    class A:
+        f = 1
+    call_optional(A(), "f")
+
 def test_nose_setup_func(testdir):
     p = testdir.makepyfile("""
         from nose.tools import with_setup
@@ -97,21 +103,13 @@ def test_nose_setup_func_failure_2(testdir):
         my_teardown = 2
 
         def test_hello():
-            print (l)
-            assert l == [1]
-
-        def test_world():
-            print (l)
-            assert l == [1,2]
+            assert l == []
 
         test_hello.setup = my_setup
         test_hello.teardown = my_teardown
     """)
-    result = testdir.runpytest(p, '-p', 'nose')
-    result.stdout.fnmatch_lines([
-        "*TypeError: 'int' object is not callable*"
-    ])
-
+    reprec = testdir.inline_run()
+    reprec.assertoutcome(passed=1)
 
 def test_nose_setup_partial(testdir):
     py.test.importorskip("functools")
@@ -282,3 +280,62 @@ def test_nose_setup_ordering(testdir):
     result.stdout.fnmatch_lines([
         "*1 passed*",
     ])
+
+
+def test_apiwrapper_problem_issue260(testdir):
+    # this would end up trying a call a optional teardown on the class
+    # for plain unittests we dont want nose behaviour
+    testdir.makepyfile("""
+        import unittest
+        class TestCase(unittest.TestCase):
+            def setup(self):
+                #should not be called in unittest testcases
+                assert 0, 'setup'
+            def teardown(self):
+                #should not be called in unittest testcases
+                assert 0, 'teardown'
+            def setUp(self):
+                print('setup')
+            def tearDown(self):
+                print('teardown')
+            def test_fun(self):
+                pass
+        """)
+    result = testdir.runpytest()
+    result.stdout.fnmatch_lines("*1 passed*")
+
+@pytest.mark.skipif("sys.version_info < (2,6)")
+def test_setup_teardown_linking_issue265(testdir):
+    # we accidentally didnt integrate nose setupstate with normal setupstate
+    # this test ensures that won't happen again
+    testdir.makepyfile('''
+        import pytest
+
+        class TestGeneric(object):
+            def test_nothing(self):
+                """Tests the API of the implementation (for generic and specialized)."""
+
+        @pytest.mark.skipif("True", reason=
+                    "Skip tests to check if teardown is skipped as well.")
+        class TestSkipTeardown(TestGeneric):
+
+            def setup(self):
+                """Sets up my specialized implementation for $COOL_PLATFORM."""
+                raise Exception("should not call setup for skipped tests")
+
+            def teardown(self):
+                """Undoes the setup."""
+                raise Exception("should not call teardown for skipped tests")
+        ''')
+    reprec = testdir.inline_run()
+    reprec.assertoutcome(passed=1, skipped=1)
+
+def test_SkipTest_during_collection(testdir):
+    testdir.makepyfile("""
+        import nose
+        raise nose.SkipTest("during collection")
+        def test_failing():
+            assert False
+        """)
+    reprec = testdir.inline_run()
+    reprec.assertoutcome(skipped=1)
