@@ -1,5 +1,5 @@
 import pytest, py, sys, os
-from _pytest import runner
+from _pytest import runner, main
 from py._code.code import ReprExceptionInfo
 
 class TestSetupState:
@@ -185,7 +185,10 @@ class BaseFunctionalTests:
                 # so we would end up calling test functions while
                 # sys.exc_info would return the indexerror
                 # from guessing the lastitem
-                assert sys.exc_info()[0] is None
+                excinfo = sys.exc_info()
+                import traceback
+                assert excinfo[0] is None, \
+                       traceback.format_exception(*excinfo)
             def teardown_function(func):
                 raise ValueError(42)
         """)
@@ -295,7 +298,7 @@ class TestSessionReports:
             class TestClass:
                 pass
         """)
-        rep = runner.pytest_make_collect_report(col)
+        rep = runner.collect_one_node(col)
         assert not rep.failed
         assert not rep.skipped
         assert rep.passed
@@ -315,7 +318,7 @@ class TestSessionReports:
             def test_func():
                 pass
         """)
-        rep = runner.pytest_make_collect_report(col)
+        rep = main.collect_one_node(col)
         assert not rep.failed
         assert not rep.passed
         assert rep.skipped
@@ -378,6 +381,10 @@ def test_runtest_in_module_ordering(testdir):
     ])
 
 
+def test_outcomeexception_exceptionattributes():
+    outcome = runner.OutcomeException('test')
+    assert outcome.args[0] == outcome.msg
+
 def test_pytest_exit():
     try:
         py.test.exit("hello")
@@ -479,3 +486,14 @@ def test_unicode_in_longrepr(testdir):
     assert result.ret == 1
     assert "UnicodeEncodeError" not in result.stderr.str()
 
+
+
+def test_failure_in_setup(testdir):
+    testdir.makepyfile("""
+        def setup_module():
+            0/0
+        def test_func():
+            pass
+    """)
+    result = testdir.runpytest("--tb=line")
+    assert "def setup_module" not in result.stdout.str()
