@@ -179,7 +179,7 @@ class TestNewSession(SessionTests):
             test_three="xxxdsadsadsadsa",
             __init__=""
         )
-        reprec = testdir.inline_run('--collectonly', p.dirpath())
+        reprec = testdir.inline_run('--collect-only', p.dirpath())
 
         itemstarted = reprec.getcalls("pytest_itemcollected")
         assert len(itemstarted) == 3
@@ -208,13 +208,14 @@ def test_plugin_specify(testdir):
             testdir.parseconfig("-p", "nqweotexistent")
     """)
     #pytest.raises(ImportError,
-    #    "config.pluginmanager.do_configure(config)"
+    #    "config.do_configure(config)"
     #)
 
 def test_plugin_already_exists(testdir):
     config = testdir.parseconfig("-p", "terminal")
     assert config.option.plugins == ['terminal']
-    config.pluginmanager.do_configure(config)
+    config.do_configure()
+    config.do_unconfigure()
 
 def test_exclude(testdir):
     hellodir = testdir.mkdir("hello")
@@ -226,3 +227,17 @@ def test_exclude(testdir):
     assert result.ret == 0
     result.stdout.fnmatch_lines(["*1 passed*"])
 
+def test_sessionfinish_with_start(testdir):
+    testdir.makeconftest("""
+        import os
+        l = []
+        def pytest_sessionstart():
+            l.append(os.getcwd())
+            os.chdir("..")
+
+        def pytest_sessionfinish():
+            assert l[0] == os.getcwd()
+
+    """)
+    res = testdir.runpytest("--collect-only")
+    assert res.ret == 0
