@@ -91,3 +91,12 @@ def test_tmpdir_always_is_realpath(testdir):
     result = testdir.runpytest("-s", p, '--basetemp=%s/bt' % linktemp)
     assert not result.ret
 
+def test_tmpdir_too_long_on_parametrization(testdir):
+    testdir.makepyfile("""
+        import pytest
+        @pytest.mark.parametrize("arg", ["1"*1000])
+        def test_some(arg, tmpdir):
+            tmpdir.ensure("hello")
+    """)
+    reprec = testdir.inline_run()
+    reprec.assertoutcome(passed=1)
