@@ -14,6 +14,26 @@ def test_simple_unittest(testdir):
     assert reprec.matchreport("testpassing").passed
     assert reprec.matchreport("test_failing").failed
 
+def test_runTest_method(testdir):
+    testpath=testdir.makepyfile("""
+        import unittest
+        pytest_plugins = "pytest_unittest"
+        class MyTestCaseWithRunTest(unittest.TestCase):
+            def runTest(self):
+                self.assertEquals('foo', 'foo')
+        class MyTestCaseWithoutRunTest(unittest.TestCase):
+            def runTest(self):
+                self.assertEquals('foo', 'foo')
+            def test_something(self):
+                pass
+        """)
+    result = testdir.runpytest("-v")
+    result.stdout.fnmatch_lines("""
+        *MyTestCaseWithRunTest.runTest*
+        *MyTestCaseWithoutRunTest.test_something*
+        *2 passed*
+    """)
+
 def test_isclasscheck_issue53(testdir):
     testpath = testdir.makepyfile("""
         import unittest
@@ -45,6 +65,45 @@ def test_setup(testdir):
     rep = reprec.matchreport("test_both", when="teardown")
     assert rep.failed and '42' in str(rep.longrepr)
 
+def test_setUpModule(testdir):
+    testpath = testdir.makepyfile("""
+        l = []
+
+        def setUpModule():
+            l.append(1)
+
+        def tearDownModule():
+            del l[0]
+
+        def test_hello():
+            assert l == [1]
+
+        def test_world():
+            assert l == [1]
+        """)
+    result = testdir.runpytest(testpath)
+    result.stdout.fnmatch_lines([
+        "*2 passed*",
+    ])
+
+def test_setUpModule_failing_no_teardown(testdir):
+    testpath = testdir.makepyfile("""
+        l = []
+
+        def setUpModule():
+            0/0
+
+        def tearDownModule():
+            l.append(1)
+
+        def test_hello():
+            pass
+    """)
+    reprec = testdir.inline_run(testpath)
+    reprec.assertoutcome(passed=0, failed=1)
+    call = reprec.getcalls("pytest_runtest_setup")[0]
+    assert not call.item.module.l
+
 def test_new_instances(testdir):
     testpath = testdir.makepyfile("""
         import unittest
@@ -76,6 +135,25 @@ def test_teardown(testdir):
     assert passed == 2
     assert passed + skipped + failed == 2
 
+@pytest.mark.skipif("sys.version_info < (2,7)")
+def test_unittest_skip_issue148(testdir):
+    testpath = testdir.makepyfile("""
+        import unittest
+
+        @unittest.skip("hello")
+        class MyTestCase(unittest.TestCase):
+            @classmethod
+            def setUpClass(self):
+                xxx
+            def test_one(self):
+                pass
+            @classmethod
+            def tearDownClass(self):
+                xxx
+    """)
+    reprec = testdir.inline_run(testpath)
+    reprec.assertoutcome(skipped=1)
+
 def test_method_and_teardown_failing_reporting(testdir):
     testdir.makepyfile("""
         import unittest, pytest
@@ -257,7 +335,27 @@ def test_testfunction_skip_property(testdir):
 
 class TestTrialUnittest:
     def setup_class(cls):
-        pytest.importorskip("twisted.trial.unittest")
+        cls.ut = pytest.importorskip("twisted.trial.unittest")
+
+    def test_trial_testcase_runtest_not_collected(self, testdir):
+        testdir.makepyfile("""
+            from twisted.trial.unittest import TestCase
+
+            class TC(TestCase):
+                def test_hello(self):
+                    pass
+        """)
+        reprec = testdir.inline_run()
+        reprec.assertoutcome(passed=1)
+        testdir.makepyfile("""
+            from twisted.trial.unittest import TestCase
+
+            class TC(TestCase):
+                def runTest(self):
+                    pass
+        """)
+        reprec = testdir.inline_run()
+        reprec.assertoutcome(passed=1)
 
     def test_trial_exceptions_with_skips(self, testdir):
         testdir.makepyfile("""
@@ -480,3 +578,97 @@ def test_unittest_unexpected_failure(testdir):
     ])
 
 
+
+def test_unittest_setup_interaction(testdir):
+    testdir.makepyfile("""
+        import unittest
+        import pytest
+        class MyTestCase(unittest.TestCase):
+            @pytest.fixture(scope="class", autouse=True)
+            def perclass(self, request):
+                request.cls.hello = "world"
+            @pytest.fixture(scope="function", autouse=True)
+            def perfunction(self, request):
+                request.instance.funcname = request.function.__name__
+
+            def test_method1(self):
+                assert self.funcname == "test_method1"
+                assert self.hello == "world"
+
+            def test_method2(self):
+                assert self.funcname == "test_method2"
+
+            def test_classattr(self):
+                assert self.__class__.hello == "world"
+    """)
+    result = testdir.runpytest()
+    result.stdout.fnmatch_lines("*3 passed*")
+
+
+def test_non_unittest_no_setupclass_support(testdir):
+    testpath = testdir.makepyfile("""
+        class TestFoo:
+            x = 0
+
+            @classmethod
+            def setUpClass(cls):
+                cls.x = 1
+
+            def test_method1(self):
+                assert self.x == 0
+
+            @classmethod
+            def tearDownClass(cls):
+                cls.x = 1
+
+        def test_not_teareddown():
+            assert TestFoo.x == 0
+
+    """)
+    reprec = testdir.inline_run(testpath)
+    reprec.assertoutcome(passed=2)
+
+
+def test_no_teardown_if_setupclass_failed(testdir):
+    testpath = testdir.makepyfile("""
+        import unittest
+
+        class MyTestCase(unittest.TestCase):
+            x = 0
+
+            @classmethod
+            def setUpClass(cls):
+                cls.x = 1
+                assert False
+
+            def test_func1(self):
+                cls.x = 10
+
+            @classmethod
+            def tearDownClass(cls):
+                cls.x = 100
+
+        def test_notTornDown():
+            assert MyTestCase.x == 1
+    """)
+    reprec = testdir.inline_run(testpath)
+    reprec.assertoutcome(passed=1, failed=1)
+
+
+def test_issue333_result_clearing(testdir):
+    testdir.makeconftest("""
+        def pytest_runtest_call(__multicall__, item):
+            __multicall__.execute()
+            assert 0
+    """)
+    testdir.makepyfile("""
+        import unittest
+        class TestIt(unittest.TestCase):
+            def test_func(self):
+                0/0
+    """)
+
+    reprec = testdir.inline_run()
+    reprec.assertoutcome(failed=1)
+
+
